//
// Shige-Fuji Project
// SwiftUI Archives
//
// Copyright (c) Watanabe-Denki, Inc.
//   https://wdkk.co.jp/
//
// This software is released under the MIT License.
//   https://opensource.org/licenses/mit-license.php
//

#include <metal_stdlib>
#include <RealityKit/RealityKit.h>

using namespace metal;

// サンプラー
constexpr sampler textureSampler(coord::normalized, address::repeat, filter::linear, mip_filter::linear);

[[visible]]
void textureSurfaceShader(realitykit::surface_parameters params)
{
    // UV座標を取得
    float2 uv = params.geometry().uv0();
    // テクスチャ座標系とY軸が逆になるので、反転した座標系を準備
    float2 inv_uv = float2( uv.x, 1.0 - uv.y );
    
    // テクスチャ画像から色をサンプリング
    auto tex = params.textures();
    half3 texColor = (half3)tex.custom().sample(textureSampler, inv_uv).rgb;
    
    // テクスチャの色に別の色を混ぜて表示してみる
    // 混ぜる色を準備
    half3 color = half3( 1.0, 1.0, 0.0 );
    // 中央の四角い領域は別の色を混ぜることにする
    if( uv.x >= 0.25 && uv.x <= 0.75 && uv.y >= 0.25 && uv.y <= 0.75 ) {
        color = half3( 0.5, 0.0, 1.0 );
    }
    
    // テクスチャの色と準備した色を混ぜる
    texColor *= color;
    // フラグメントの色を設定
    params.surface().set_base_color(texColor);
}


